@@ -2,7 +2,7 @@
 PC-BASIC - codepage.py
 Codepage conversions
 
-(c) 2013--2020 Rob Hagemans
+(c) 2013--2021 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -12,11 +12,10 @@ import codecs
 import os
 import io
 
-
 from ..compat import iterchar, iteritems, int2byte, unichr
 
 from .base.codestream import StreamWrapper
-
+from .data import DEFAULT_CODEPAGE
 
 # characters in the printable ASCII range 0x20-0x7E cannot be redefined
 # but can have their glyphs subsituted - they will work and transcode as the
@@ -28,21 +27,6 @@ PRINTABLE_ASCII = tuple(int2byte(_c) for _c in range(0x20, 0x7F))
 # BEL, TAB, LF, HOME, CLS, CR, RIGHT, LEFT, UP, DOWN  (and not BACKSPACE)
 CONTROL = (b'\x07', b'\x09', b'\x0A', b'\x0B', b'\x0C', b'\x0D', b'\x1C', b'\x1D', b'\x1E', b'\x1F')
 
-# default is codepage 437
-DEFAULT_CODEPAGE = {int2byte(_i): _c for _i, _c in enumerate(
-    u'\x00\u263a\u263b\u2665\u2666\u2663\u2660\u2022\u25d8\u25cb\u25d9\u2642\u2640\u266a\u266b'
-    u'\u263c\u25ba\u25c4\u2195\u203c\xb6\xa7\u25ac\u21a8\u2191\u2193\u2192\u2190\u221f\u2194\u25b2'
-    u'\u25bc !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrst'
-    u'uvwxyz{|}~\u2302\xc7\xfc\xe9\xe2\xe4\xe0\xe5\xe7\xea\xeb\xe8\xef\xee\xec\xc4\xc5\xc9\xe6\xc6'
-    u'\xf4\xf6\xf2\xfb\xf9\xff\xd6\xdc\xa2\xa3\xa5\u20a7\u0192\xe1\xed\xf3\xfa\xf1\xd1\xaa\xba\xbf'
-    u'\u2310\xac\xbd\xbc\xa1\xab\xbb\u2591\u2592\u2593\u2502\u2524\u2561\u2562\u2556\u2555\u2563'
-    u'\u2551\u2557\u255d\u255c\u255b\u2510\u2514\u2534\u252c\u251c\u2500\u253c\u255e\u255f\u255a'
-    u'\u2554\u2569\u2566\u2560\u2550\u256c\u2567\u2568\u2564\u2565\u2559\u2558\u2552\u2553\u256b'
-    u'\u256a\u2518\u250c\u2588\u2584\u258c\u2590\u2580\u03b1\xdf\u0393\u03c0\u03a3\u03c3\xb5\u03c4'
-    u'\u03a6\u0398\u03a9\u03b4\u221e\u03c6\u03b5\u2229\u2261\xb1\u2265\u2264\u2320\u2321\xf7\u2248'
-    u'\xb0\u2219\xb7\u221a\u207f\xb2\u25a0\xa0'
-)}
-
 # left-connecting box drawing chars for two box-character sets (single line, double line)
 _BOX_LEFT_UNICODE = (u'\u2500', u'\u2550')
 
@@ -96,7 +80,10 @@ class Codepage(object):
 
     def __init__(self, codepage_dict=None, box_protect=True):
         """Load and initialise codepage tables."""
-        codepage_dict = codepage_dict or DEFAULT_CODEPAGE
+        codepage_dict = codepage_dict or {
+            int2byte(_i): _c
+            for _i, _c in enumerate(DEFAULT_CODEPAGE)
+        }
         # protect box drawing sequences under dbcs?
         self.box_protect = box_protect
         # lead and trail bytes
