@@ -2,7 +2,7 @@
 PC-BASIC - operators.py
 Numeric and string operators
 
-(c) 2013--2020 Rob Hagemans
+(c) 2013--2021 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -13,30 +13,32 @@ from .. import values
 # operators and precedence
 # key is tuple (token, nargs)
 PRECEDENCE = {
-    tk.O_CARET: 12,
-    tk.O_TIMES: 11,
-    tk.O_DIV: 11,
-    tk.O_INTDIV: 10,
-    tk.MOD: 9,
-    tk.O_PLUS: 8,
-    tk.O_MINUS: 8,
-    tk.O_GT: 7,
-    tk.O_EQ: 7,
-    tk.O_LT: 7,
-    tk.O_GT + tk.O_EQ: 7,
-    tk.O_EQ + tk.O_GT: 7,
-    tk.O_LT + tk.O_EQ: 7,
-    tk.O_EQ + tk.O_LT: 7,
-    tk.O_LT + tk.O_GT: 7,
-    tk.O_GT + tk.O_LT: 7,
-    tk.NOT: 6,
-    tk.AND: 5,
-    tk.OR: 4,
-    tk.XOR: 3,
-    tk.EQV: 2,
-    tk.IMP: 1,
+    (tk.O_CARET, 2): 13,
+    (tk.O_PLUS, 1): 12,
+    (tk.O_MINUS, 1): 12,
+    (tk.O_TIMES, 2): 11,
+    (tk.O_DIV, 2): 11,
+    (tk.O_INTDIV, 2): 10,
+    (tk.MOD, 2): 9,
+    (tk.O_PLUS, 2): 8,
+    (tk.O_MINUS, 2): 8,
+    (tk.O_GT, 2): 7,
+    (tk.O_EQ, 2): 7,
+    (tk.O_LT, 2): 7,
+    (tk.O_GT + tk.O_EQ, 2): 7,
+    (tk.O_EQ + tk.O_GT, 2): 7,
+    (tk.O_LT + tk.O_EQ, 2): 7,
+    (tk.O_EQ + tk.O_LT, 2): 7,
+    (tk.O_LT + tk.O_GT, 2): 7,
+    (tk.O_GT + tk.O_LT, 2): 7,
+    (tk.NOT, 1): 6,
+    (tk.AND, 2): 5,
+    (tk.OR, 2): 4,
+    (tk.XOR, 2): 3,
+    (tk.EQV, 2): 2,
+    (tk.IMP, 2): 1,
 }
-OPERATORS = set(PRECEDENCE)
+OPERATORS = set(operator_arity[0] for operator_arity in PRECEDENCE)
 
 # can be combined like <> >=
 COMBINABLE = (tk.O_LT, tk.O_EQ, tk.O_GT)
